// All database related API calls

import { useAuth } from '../authorization';
import { useAppDispatch, useSessionCache } from '../store';
import { updateCurrentDatabase, updateDatabaseList } from '../store/sessionSlice';

export enum DatabaseType {
  ArangoDB = 0,
  Neo4j = 1,
}

export const databaseNameMapping: string[] = ['arangodb', 'neo4j'];
export const databaseProtocolMapping: string[] = ['neo4j://', 'neo4j+s://', 'bolt://', 'bolt+s://'];

export type AddDatabaseRequest = {
  name: string;
  internal_database_name: string;
  url: string;
  port: number;
  protocol: string;
  username: string;
  password: string;
  type: DatabaseType; // Database type. 0 = ArangoDB, 1 = Neo4j
};

export type DatabaseInfo = {
  Name: string;
  InternalDatabaseName: string;
  URL: string;
  Port: number;
  Protocol: string;
  Username: string;
  Password: string;
  Type: number;
  status: boolean;
};

export type AddDatabaseOptions = {
  setAsCurrent?: boolean;
  updateDatabaseCache?: boolean;
};

export type GetDatabasesOptions = {
  updateSessionCache?: boolean;
};

export type DeleteDatabasesOptions = {
  updateSessionCache?: boolean;
};

export type VerifyConnectionRequest = string;

export const useDatabaseAPI = () => {
  const cache = useSessionCache();
  const dispatch = useAppDispatch();
  const domain = import.meta.env.BACKEND_URL;
  const useruri = import.meta.env.BACKEND_USER;
  const { fetchAuthenticated } = useAuth();

  function AddDatabase(request: AddDatabaseRequest, options: AddDatabaseOptions = {}): Promise<void> {
    const { setAsCurrent = true, updateDatabaseCache = false } = options;
    return new Promise((resolve, reject) => {
      fetchAuthenticated(`${domain}${useruri}/database`, {
        method: 'POST',
        body: JSON.stringify(request),
      })
        .then((response: Response) => {
          console.info('Added Database', response);
          if (!response.ok) {
            reject(response.statusText);
          }
          if (setAsCurrent) dispatch(updateCurrentDatabase(request.name));
          if (updateDatabaseCache) GetAllDatabases({ updateSessionCache: true }).catch(reject);

          resolve();
        })
        .catch(reject);
    });
  }

  function GetAllDatabases(options: GetDatabasesOptions = {}): Promise<Array<string>> {
    const { updateSessionCache: updateDatabaseCache = true } = options;
    return new Promise((resolve, reject) => {
      fetchAuthenticated(`${domain}${useruri}/database`)
        .then((response: Response) => {
          if (!response.ok) {
            new Promise((resolve, reject) => {
              reject(response.statusText);
            });
          }
          response
            .json()
            .then((json) => {
              if (updateDatabaseCache) dispatch(updateDatabaseList(json.databases));
              resolve(json.databases);
            })
            .catch(reject);
        })
        .catch(reject);
    });
  }

  function DeleteDatabase(name: string, options: DeleteDatabasesOptions = {}): Promise<void> {
    const { updateSessionCache: updateDatabaseCache = true } = options;
    return new Promise((resolve, reject) => {
      fetchAuthenticated(`${domain}${useruri}/database/` + name, {
        method: 'DELETE',
      })
        .then((response: Response) => {
          if (!response.ok) {
            reject(response.statusText);
          }

          if (updateDatabaseCache) GetAllDatabases({ updateSessionCache: true }).catch(reject);

          resolve();
        })
        .catch(reject);
    });
  }

  function TestDatabaseConnection(request: AddDatabaseRequest): Promise<void> {
    return new Promise((resolve, reject) => {
      fetchAuthenticated(`${domain}${useruri}/database/test-connection`, {
        method: 'POST',
        body: JSON.stringify(request),
      })
        .then((response: Response) => {
          if (!response.ok) {
            reject(response.statusText);
          }
          resolve(response.json());
        })
        .catch(reject);
    });
  }

  function UpdateDatabase(request: AddDatabaseRequest): Promise<void> {
    return new Promise((resolve, reject) => {
      fetchAuthenticated(`${domain}${useruri}/database/update`, {
        method: 'PATCH',
        body: JSON.stringify(request),
      })
        .then((response: Response) => {
          if (!response.ok) {
            reject(response.statusText);
          }
          resolve();
        })
        .catch(reject);
    });
  }

  return {
    DatabaseType,
    AddDatabase,
    GetAllDatabases,
    DeleteDatabase,
    TestDatabaseConnection,
    UpdateDatabase,
  };
};
